VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "MapInfoCreator"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
'
' Export MapPoint shapes and pushpins on the active map into MapInfo mif/mid files.
' Date: 11/1/2001
' Author: Walt Cygan
' (wcygan@macrogroup.net)
'
' This code is copyrighted freeware - use freely, but please leave this
' header intact. Suggestions and comments welcome.
'
Private m_objMap As MapPoint.Map
Private m_strFileName As String
Private m_colMIDLines As Collection
Private m_colMIFLines As Collection
Private m_lObjectCount As Long
Private m_booSomeNotExported As Boolean

Private Sub Class_Initialize()
    'When the class is instantiated, create the collections that will hold the
    'text lines that will be written to the mif/mid files later.
    Set m_colMIDLines = New Collection
    Set m_colMIFLines = New Collection
End Sub

Private Sub Class_Terminate()
    'When the class is terminated, destroy the objects created.
    Set m_colMIDLines = Nothing
    Set m_colMIFLines = Nothing
    Set m_objMap = Nothing
End Sub

Public Property Set Map(objMap As MapPoint.Map)
    'Set the map that will be used throughout this process to what was passed from the form.
    Set m_objMap = objMap
End Property

Public Property Let FileName(strFileName As String)
    'Set the file name to what was retrieved from the common dialog.
    m_strFileName = strFileName
End Property

Public Sub CreateMapInfoFiles()
On Error GoTo ErrSub

    'Initialize the object count.
    m_lObjectCount = 1
    
    'Create the mif file header.
    UpdateStatus "Shape Export Started"
    CreateMIFHeader
    
    'Create the mif file body and the mid file.
    UpdateStatus "  ...MIF Header Rows Written"
    CreateMIFBody m_objMap
    
    'Write the files to disk.
    UpdateStatus "  ...MIF Shape Rows Written"
    UpdateStatus "  ...Preparing to Create Export Files"
    WriteMIFiles
    
    'Finish updating the status.
    UpdateStatus "  ...Export Files Created"
    If m_booSomeNotExported Then
        UpdateStatus "NOTE: Not all shapes could be exported."
        UpdateStatus "  ...Text boxes cannot be exported."
    End If
    UpdateStatus "Shape Export Completed"
    
ExitSub:
    Exit Sub
ErrSub:
    UpdateStatus "Error reported in '" & Err.Source & "' procedure: " & CStr(Err.Number) & ": " & Err.Description
    Err.Raise Err.Number, IIf(Err.Source = "SpatialDataImport", "CreateMapInfoFiles", Err.Source), Err.Description
    Resume ExitSub
End Sub

Private Sub CreateMIFHeader()
On Error GoTo ErrSub

    'Create the static mif header.
    m_colMIFLines.Add "Version 300" & vbCrLf
    m_colMIFLines.Add "Charset " & Chr$(34) & "WindowsLatin1" & Chr$(34) & vbCrLf
    m_colMIFLines.Add "Delimiter " & Chr$(34) & "," & Chr$(34) & vbCrLf
    m_colMIFLines.Add "CoordSys Earth Projection 1, 0" & vbCrLf
    m_colMIFLines.Add "Columns 5" & vbCrLf
    m_colMIFLines.Add "  Name Char(30)" & vbCrLf
    m_colMIFLines.Add "  ID Integer" & vbCrLf
    m_colMIFLines.Add "  ShapeProperty Long" & vbCrLf
    m_colMIFLines.Add "  ShapeProperty2 Long" & vbCrLf
    m_colMIFLines.Add "  ShapeText Char(255)" & vbCrLf
    m_colMIFLines.Add "Data" & vbCrLf
    m_colMIFLines.Add " " & vbCrLf

ExitSub:
    Exit Sub
ErrSub:
    Err.Raise Err.Number, IIf(Err.Source = "SpatialDataImport", "CreateMIFHeader", Err.Source), Err.Description
    Resume ExitSub
End Sub

Private Sub CreateMIFBody(ByRef objMap As MapPoint.Map)
On Error GoTo ErrSub
Dim objShape As MapPoint.Shape

    For Each objShape In objMap.Shapes
        'A line and a region really look the same programatically, so I've declared that
        'a filled object is a region and a non-filled object is a line.
        If objShape.Type = geoFreeform And objShape.Fill.Visible = False Then
            'Output a LINE.
            PLine objShape
        ElseIf objShape.Type = geoFreeform And objShape.Fill.Visible = True Then
            'Output a REGION.
            Region objShape
        ElseIf objShape.Type = geoAutoShape Then
            If objShape.AutoShapeType = geoShapeRectangle Then
                'Output a RECTANGLE.
                Rectangle objShape
            Else
                'Something that can't be handled
                m_booSomeNotExported = True
            End If
        Else
            'Something that can't be handled
            m_booSomeNotExported = True
        End If
    Next
    'Output POINTS (pushpins).
    Points

ExitSub:
    Set objShape = Nothing
    Exit Sub
ErrSub:
    Err.Raise Err.Number, IIf(Err.Source = "SpatialDataImport", "CreateMIFBody", Err.Source), Err.Description
    Resume ExitSub
End Sub

Private Sub WriteMIFiles()
On Error GoTo ErrSub
Dim strMIDFileName As String

    strMIDFileName = m_strFileName
    'Get the mid file name.
    Mid(strMIDFileName, Len(strMIDFileName), 1) = "d"
    'Write the mif file to disk.
    WriteTextLines m_strFileName, m_colMIFLines
    'Write the mid file to disk.
    WriteTextLines strMIDFileName, m_colMIDLines

ExitSub:
    Exit Sub
ErrSub:
    Err.Raise Err.Number, IIf(Err.Source = "SpatialDataImport", "WriteMIFiles", Err.Source), Err.Description
    Resume ExitSub
End Sub

Private Sub WriteTextLines(sFilename As String, m_colLines As Collection)
On Error GoTo ErrSub
Dim osFSO As Scripting.FileSystemObject
Dim osTextStream As Scripting.TextStream
Dim lArrayIndex As Long
Dim vTextLine As Variant
    
    'Create the needed file objects.
    Set osFSO = New Scripting.FileSystemObject
    Set osTextStream = osFSO.OpenTextFile(sFilename, ForWriting, True, TristateFalse)
    
    'Write the collection of print lines to the file.
    For Each vTextLine In m_colLines
        osTextStream.Write vTextLine
    Next
    
    'Close the file.
    osTextStream.Close
    
ExitSub:
    'Clean up.
    Set osFSO = Nothing
    Set osTextStream = Nothing
    Exit Sub
ErrSub:
    Err.Raise Err.Number, IIf(Err.Source = "SpatialDataImport", "WriteTextLines", Err.Source), Err.Description
    Resume ExitSub
End Sub

Private Sub PLine(ByRef objShape As MapPoint.Shape)
On Error GoTo ErrSub
Dim iVertices As Integer
Dim dblLat As Double
Dim dblLon As Double
    
    'Add the Pline start line.
    m_colMIFLines.Add "Pline " & CStr(UBound(objShape.Vertices) + 1) & vbCrLf
    'For each point on the line, get the lat/lon, and add a file line.
    For iVertices = 0 To UBound(objShape.Vertices)
        CalcPos m_objMap, objShape.Vertices(iVertices).Location, dblLat, dblLon
        m_colMIFLines.Add CStr(Round(dblLon, 6)) & " " & CStr(Round(dblLat, 6)) & vbCrLf
    Next
    'Add the pen line.
    m_colMIFLines.Add "    Pen (1,2,13408665) " & vbCrLf
    
    'Add a data line to the mid file, including the line weight.
    m_colMIDLines.Add Chr$(34) & "Line_" & CStr(m_lObjectCount) & Chr$(34) & "," & CStr(m_lObjectCount) & "," & CStr(objShape.Line.ForeColor) & "," & CStr(objShape.Line.Weight) & "," & Chr$(34) & objShape.Name & Chr$(34) & vbCrLf
    
    'Update the object count.
    frmSpatialDataImport.lblRecordCount.Caption = "Shape Count: " & m_lObjectCount
    DoEvents
    m_lObjectCount = m_lObjectCount + 1

ExitSub:
    Exit Sub
ErrSub:
    Err.Raise Err.Number, IIf(Err.Source = "SpatialDataImport", "PLine", Err.Source), Err.Description
    Resume ExitSub
End Sub

Private Sub Region(ByRef objShape As MapPoint.Shape)
On Error GoTo ErrSub
Dim iVertices As Integer
Dim dblLat As Double
Dim dblLatHigh As Double
Dim dblLatLow As Double
Dim dblLon As Double
Dim dblLonHigh As Double
Dim dblLonLow As Double
            
    'Add the Region start line.
    m_colMIFLines.Add "Region  1" & vbCrLf
    m_colMIFLines.Add "  " & CStr(UBound(objShape.Vertices) + 2) & vbCrLf
    
    'Initialize variable to allow the lat long of the center point to be calculated.
    dblLatHigh = -999
    dblLatLow = 999
    dblLonHigh = -999
    dblLonLow = 999
    
    'For each point on the region's border, get the lat/lon, and add a file line.
    'Then see if the min and max values of the lat and lon are affected.
    For iVertices = 0 To UBound(objShape.Vertices)
        CalcPos m_objMap, objShape.Vertices(iVertices).Location, dblLat, dblLon
        m_colMIFLines.Add CStr(Round(dblLon, 6)) & " " & CStr(Round(dblLat, 6)) & vbCrLf
        dblLatHigh = IIf(dblLat > dblLatHigh, dblLat, dblLatHigh)
        dblLatLow = IIf(dblLat < dblLatLow, dblLat, dblLatLow)
        dblLonHigh = IIf(dblLon > dblLonHigh, dblLon, dblLonHigh)
        dblLonLow = IIf(dblLon < dblLonLow, dblLon, dblLonLow)
    Next
    'Add the closing point of the region, which is the same as the first point.
    CalcPos m_objMap, objShape.Vertices(0).Location, dblLat, dblLon
    m_colMIFLines.Add CStr(Round(dblLon, 6)) & " " & CStr(Round(dblLat, 6)) & vbCrLf
    'Add the pen, brush and center lines.
    m_colMIFLines.Add "    Pen (1,1,0) " & vbCrLf
    m_colMIFLines.Add "    Brush (2,10079385,16777215) " & vbCrLf
    m_colMIFLines.Add "    Center " & CStr(Round((dblLonHigh + dblLonLow) / 2, 6)) & " " & CStr(Round((dblLatHigh + dblLatLow) / 2, 6)) & " " & vbCrLf
    
    'Add a data line to the mid file, including the line weight and fill color.
    m_colMIDLines.Add Chr$(34) & "Region_" & CStr(m_lObjectCount) & Chr$(34) & "," & CStr(m_lObjectCount) & "," & CStr(objShape.Fill.ForeColor) & "," & CStr(objShape.Line.Weight) & "," & Chr$(34) & objShape.Name & Chr$(34) & vbCrLf
    
    'Update the object count.
    frmSpatialDataImport.lblRecordCount.Caption = "Shape Count: " & m_lObjectCount
    DoEvents
    m_lObjectCount = m_lObjectCount + 1
    
ExitSub:
    Exit Sub
ErrSub:
    Err.Raise Err.Number, IIf(Err.Source = "SpatialDataImport", "Region", Err.Source), Err.Description
    Resume ExitSub
End Sub

Private Sub Rectangle(ByRef objShape As MapPoint.Shape)
On Error GoTo ErrSub
Dim iVertices As Integer
Dim dblLatC As Double
Dim dblLonC As Double
Dim dblLatLL As Double
Dim dblLonLL As Double
Dim dblLatUR As Double
Dim dblLonUR As Double
Dim dblLatLeft As Double
Dim dblLatRight As Double
Dim dblLatTop As Double
Dim dblLatBottom As Double
Dim dblLonLeft As Double
Dim dblLonRight As Double
Dim dblLonTop As Double
Dim dblLonBottom As Double
Dim objLocBottom As MapPoint.Location
Dim objLocTop As MapPoint.Location
Dim objLocLeft As MapPoint.Location
Dim objLocRight As MapPoint.Location
Dim dblDistanceHeight As Double
Dim dblDistanceWidth As Double

Dim XC As Long
Dim YC As Long
            
    'Rectangles are a bit more difficult:
    '   Center the object
    objShape.Location.GoTo
    '   Get the lat/lon of the center's location object.
    CalcPos m_objMap, objShape.Location, dblLatC, dblLonC
    
    'Get the x and y screen coordinates of the center.
    XC = m_objMap.LocationToX(objShape.Location)
    YC = m_objMap.LocationToY(objShape.Location)
    'Get the lat and lon of locations at the top, bottom, right and left of the screen.
    Set objLocTop = m_objMap.XYToLocation(XC, 0)
    CalcPos m_objMap, objLocTop, dblLatTop, dblLonTop
    Set objLocLeft = m_objMap.XYToLocation(0, YC)
    CalcPos m_objMap, objLocLeft, dblLatLeft, dblLonLeft
    Set objLocBottom = m_objMap.XYToLocation(XC, m_objMap.Height)
    CalcPos m_objMap, objLocBottom, dblLatBottom, dblLonBottom
    Set objLocRight = m_objMap.XYToLocation(m_objMap.Width, YC)
    CalcPos m_objMap, objLocRight, dblLatRight, dblLonRight
    
    'Get the vertical and horizontance distance (miles or km) of the displayed map.
    dblDistanceHeight = objLocTop.DistanceTo(objLocBottom)
    dblDistanceWidth = objLocLeft.DistanceTo(objLocRight)

    'Get the lat/lon of the lower left and upper right corners
    dblLonLL = dblLonC - ((objShape.Width / dblDistanceWidth) * (dblLonC - dblLonLeft))
    dblLatLL = dblLatC - ((objShape.Height / dblDistanceHeight) * (dblLatC - dblLatBottom))
    dblLonUR = dblLonC - ((objShape.Width / dblDistanceWidth) * (dblLonC - dblLonRight))
    dblLatUR = dblLatC - ((objShape.Height / dblDistanceHeight) * (dblLatC - dblLatTop))
    
    'Add the Rectangle start, pen and brush lines.
    m_colMIFLines.Add "Rect " & CStr(Round(dblLonLL, 6)) & " " & CStr(Round(dblLatLL, 6)) & " " & CStr(Round(dblLonUR, 6)) & " " & CStr(Round(dblLatUR, 6)) & vbCrLf
    m_colMIFLines.Add "    Pen (1,2,0) " & vbCrLf
    m_colMIFLines.Add "    Brush (2,16777215,16777215) " & vbCrLf
    
    'Add a data line to the mid file, including the line weight and fill color..
    m_colMIDLines.Add Chr$(34) & "Rect_" & CStr(m_lObjectCount) & Chr$(34) & "," & CStr(m_lObjectCount) & "," & CStr(objShape.Line.ForeColor) & "," & CStr(objShape.Line.Weight) & "," & Chr$(34) & objShape.Name & Chr$(34) & vbCrLf
    
    'Update the object count.
    frmSpatialDataImport.lblRecordCount.Caption = "Shape Count: " & m_lObjectCount
    DoEvents
    m_lObjectCount = m_lObjectCount + 1

ExitSub:
    'Clean up.
    Set objLocBottom = Nothing
    Set objLocTop = Nothing
    Set objLocLeft = Nothing
    Set objLocRight = Nothing
    Exit Sub
ErrSub:
    Err.Raise Err.Number, IIf(Err.Source = "SpatialDataImport", "Rectangle", Err.Source), Err.Description
    Resume ExitSub
End Sub

Private Sub Points()
On Error GoTo ErrSub
Dim objDataSet As MapPoint.DataSet
Dim objRecordset As MapPoint.Recordset
Dim objField As MapPoint.Field
Dim dblLat As Double
Dim dblLon As Double
Dim strNote As String

    'For each pushpin, output the lat/lon and symbol lines.
    For Each objDataSet In m_objMap.DataSets
        Set objRecordset = objDataSet.QueryAllRecords
        objRecordset.MoveFirst
        Do Until objRecordset.EOF
            CalcPos m_objMap, objRecordset.Pushpin.Location, dblLat, dblLon
            m_colMIFLines.Add "Point " & CStr(Round(dblLon, 6)) & " " & CStr(Round(dblLat, 6)) & vbCrLf
            m_colMIFLines.Add "    Symbol (44,13395711,10)" & vbCrLf
            strNote = ""
            'Add information in the balloon and note fields to the data to be written.
            For Each objField In objRecordset.Fields
                If objField.VisibleInBalloon Then
                    strNote = strNote & objField.Name & ": " & CStr(objField.Value) & "> "
                End If
            Next
            If Len(strNote) >= 2 Then
                strNote = Left$(strNote, (Len(strNote) - 2))
            End If
            If objRecordset.Pushpin.Note <> "" Then
                strNote = strNote & IIf(strNote <> "", "> Note: ", "") & objRecordset.Pushpin.Note
            End If
            
            'Add a data line to the mid file, including the pushpin data and symbol.
            m_colMIDLines.Add Chr$(34) & objRecordset.Pushpin.Name & Chr$(34) & "," & CStr(m_lObjectCount) & "," & objRecordset.Pushpin.Symbol & ",0," & Chr$(34) & Left$(strNote, 255) & Chr$(34) & vbCrLf
    
            'Update the object count.
            frmSpatialDataImport.lblRecordCount.Caption = "Shape Count: " & m_lObjectCount
            DoEvents
            m_lObjectCount = m_lObjectCount + 1
            objRecordset.MoveNext
        Loop
    Next

ExitSub:
    'Clean up.
    Set objField = Nothing
    Set objRecordset = Nothing
    Set objDataSet = Nothing
    Exit Sub
ErrSub:
    Err.Raise Err.Number, IIf(Err.Source = "SpatialDataImport", "Points", Err.Source), Err.Description
    Resume ExitSub
End Sub
